/**********************************************************
           XTEA - Tiny Encryption Algorithm

     Unrolled, tweaked by A.Myasnikov, DarkSoftware

             Web: www.darksoftware.narod.ru

************************************************************/



#include <string.h>


typedef unsigned long u32;


u32 key[4];

typedef struct
{
  u32 sum0;
  u32 sum1;
  u32 csum0;
  u32 csum1;
} sumrec;

sumrec precomp[38];


void
xtea_encipher (u32 * v)
{
  u32 v0 = v[0], v1 = v[1], i;
  for (i = 0; i != 38; i++)
    {
      v0 +=
        (((v1 << 6) ^ (v1 >> 9)) + v1) ^ (precomp[i].sum0 + precomp[i].csum0);
      v1 +=
        (((v0 << 6) ^ (v0 >> 9)) + v0) ^ (precomp[i].sum1 + precomp[i].csum1);
    }
  v[0] = v0;
  v[1] = v1;
}

void
xtea_decipher (u32 * v)
{
  u32 v0 = v[0], v1 = v[1];
  long i;
  for (i = 37; i != -1; i--)
    {
      v1 -=
        (((v0 << 6) ^ (v0 >> 9)) + v0) ^ (precomp[i].sum1 + precomp[i].csum1);
      v0 -=
        (((v1 << 6) ^ (v1 >> 9)) + v1) ^ (precomp[i].sum0 + precomp[i].csum0);
    }
  v[0] = v0;
  v[1] = v1;
}




void __export __stdcall
setup (u32 * data)
{
  u32 sum = 0, delta = 0x9E3779B9, i;
  memcpy (&key[0], &data[0], 16);


  for (i = 0; i < 38; i++)
    {
      precomp[i].sum0 = sum;
      precomp[i].csum0 = key[(sum & 3)];
      sum += delta;
      precomp[i].sum1 = sum;
      precomp[i].csum1 = key[((sum >> 11) & 3)];
    }

}



void __export __stdcall
crypt (u32 * data)
{
  xtea_encipher (data);
}

void __export __stdcall
decrypt (u32 * data)
{
  xtea_decipher (data);
}
